%******************************************
%ABSENTEE.M 

%This is a Matlab program.
%It replicates the empirical work reported in my paper
%"Recounts from Undervotes: Evidence from the 2001 Presidential Election"
%September, 2001
%Revised: April, 2002

%The program calls the companion file "beta.prc", which should be in the same directory.
%It loads the data file "data.txt", and creates an output file "beta.out".  

%This program could be modified to estimate datasets using the multivariate BetaLogit likelihood.

%The program uses the Gauss package "OPTMUM" and call Gauss graphics, so both libraries must be active. 
%That is, issue the command
%library pgraph,optmum;

%Bruce Hansen
%bhansen@ssc.wisc.edu
%http://www.ssc.wisc.edu/~bhansen/

%*****************************************/
function absentee()
global xc;
out=fopen('absenteeout.txt','wt');
% Data %
n=282;
load absenteedat.txt;
dat=absenteedat;

% ***************************************** %
pn=2;				% number of models @
y2=dat(:,[5 6]);
y2=y2.*(y2>0);
ballots=dat(:,[3 4])+y2;
xc=ones(n,1);x=xc;
xcnames = '_Const';xnames=xcnames;pnames=xcnames;

% Creat the data files %
y=dat(:,[3 4]);			% first model - dependent variables @
titlee='Gore Percentage in Machine-Readable Vote';
xselect =0;		
pselect = 0 ;		% index of latent probabilities as covariates @
ipselect = 0;
save('dat1.mat','y','titlee','xselect','pselect','ipselect');

y=dat(:,[5 6]);
y=y.*(y>0);
titlee='Gore Percentage in Recount';
xselect =0;	
pselect = 0 ;
ipselect = 1;
save('dat2.mat','y','titlee','xselect','pselect','ipselect');



% Estimation %
%options = optimset('LargeScale','on','GradObj','on','JacobMult',@G_Like_Beta,'HessMult',@H_Like_Beta);
options = optimset('GradObj','on','LargeScale','on','TolFun',0.0000000000001);
p=zeros(n,pn);
e=zeros(n,pn);

rt=zeros(pn,1);
global x_;global y_;global w_;
j=1;
while j<=pn
    nj=num2str(j);
    load(strcat('dat',nj,'.mat'));
    y_=y;
    w_=((y_(:,1)+y_(:,2))>0);
    if sum(xselect)>0; 
        xs=x(:,xselect); 
    else; 
        xs=0; 
    end;
    x_=xp(xs,p,j);
    phat=(sum(y_(:,1).*w_)/sum(sum(y_.*(w_*ones(1,length(y_(1,:)))))))';
    theta0=[zeros(length(x_(1,:)),1);(1/phat/4)];
    theta0(1)=ilogit(phat);
     
    [theta,f]= fminunc(@Like_Beta,theta0,options);
    Like_Beta(theta)
    g=V_G_Like_Beta(theta,y_,x_);
    gr=sum(g);
    h=H_Like_Beta(theta);
    p(:,j) = pr_hat(y_,x_,theta);
    e(:,j)=(y_(:,1)./(y_(:,1)+y_(:,2)+(1-w_))-logit(x_*theta(1:length(x_(1,:))))).*w_;
    save(strcat('dat',nj,'.mat'),'phat','x_','xs','w_','theta','f','g','gr','h','-append'),
    rt(j)=length(theta);
    j=j+1;
end;

%*****************************************%
%Covariance Matrix%
r=sum(rt);
hh=zeros(r,r);
load(strcat('dat1.mat'));
gg=g;
crt=[0;cumsum(rt)];
j1=1;
while j1<=pn
    rj1=rt(j1);
    nj=num2str(j1);
    if j1>1;
        load(strcat('dat',nj,'.mat'));
        gg=[gg,g];
    end;
    hh(crt(j1)+1:crt(j1)+rj1,crt(j1)+1:crt(j1)+rj1)=h;
    ttheta=theta;
    j2=1;
    while j2<=rj1
        thetaj2=ttheta(j2);
        ttheta
        ax=abs(thetaj2);
        delta=(ax>1).*ax + (ax<=1).*(10^(-6));
        theta(j2)=thetaj2+delta;
        pj=p;
        xsxs=xp(xs,pj,j1);
        pj(:,j1)=pr_hat(y,xsxs,ttheta);
        j3=j1+1;
        while j3<=pn
            nj3=num2str(j3);
            load(strcat('dat',nj3,'.mat'));
            xsxs=xp(xs,pj,j3);
            thetaj3=theta;
            pj(:,j3)=pr_hat(y,xsxs,thetaj3);
            hh(crt(j1)+j2,crt(j3)+1:crt(j3+1))=(sum(V_G_Like_Beta(thetaj3,y,xsxs))-gr)./delta;
            j3=j3+1;
        end;
        load(strcat('dat',nj,'.mat'));
        ttheta(j2)=thetaj2;
        j2=j2+1;
    end;
    j1=j1+1;
end;
v=inv(hh);

v=v'*(gg'*gg)*v;

%******************************%
% Print Results %
rr=0;
j=1;
while j<=pn
    nj=num2str(j);
    load(strcat('dat',nj,'.mat'));
    tit=titlee;
    rj=length(theta);
    ij=(rr+1:rj+rr)';vj=v(ij,ij);
    save(strcat('dat',nj,'.mat'),'vj','-append'),
    rr=rr+rj;
    names=xcnames;
    if sum(xselect)>0;
        names=[names;xnames(xselect,:)];
    end;
    if sum(pselect)>0;
        names=[names;pnames(pselect,:)];
    end;
    if ipselect==1
        names=[names;xcnames];
    end;
    s=theta(rj);
    sig=sqrt(phat.*(1-phat)./(1+s));
    th=[theta;sig];
    se=sqrt(diag(vj));
    se=[se;(se(rj)*sig/(2*(1+s)))];
    fprintf(out,'%s\n',tit);
    fprintf(out,'Number of Precints %f\n',sum(w_));
    fprintf(out,'Totals             %f\n',sum(y));
    fprintf(out,'Percentage         %f\n\n',phat);
    fprintf(out,'Variable   Estimate   St Error\n');
    prnames=[names;'_Scale';'____SD'];
    for ii=1:length(th)
        fprintf(out,'%s   %f     %f\n',prnames(ii,:),th(ii),se(ii));
    end;
    fprintf(out,'Negative Log Likelihood ');
    fprintf(out,'%f\n\n\n******************\n',f); 
    j=j+1;
end;
fclose(out);

%*****************************************/
%  Plot Graphs @
npoints=199;
xpoints=(1/(npoints+1):1/(npoints+1):npoints/(npoints+1))';

j1=1;
while j1<=pn
    nj1=num2str(j1);
    load(strcat('dat',nj1,'.mat'));
    if sum(ipselect)~=0;
        p1=ilogit(xpoints);
        if pselect~=0
            p2=(sum(log(p(:,pselect)).*(w_*ones(1,length(pselect))).*(ballots*ones(1,length(pselect))))./sum(w_.*ballots))';
        end;
        v=vj;
        v=v(1:length(v(:,1))-1,1:length(v(:,1))-1);
        tt=1;
        tn=1;
        z1=ones(npoints,1);
        if sum(xselect)>0
            cx=length(xselect(:,1));
            tt=[tt;(tn+1:cx+tn)'];
            tn=tn+cx;
            z1=[z1,ones(npoints,cx)];
        end;
        if sum(pselect)>0
            tt=[tt;(tn+1:length(pselect(:,1))+tn)'*[1,1,1]];
            tn=tn+length(pselect(:,1));
            z1=[z1,ones(npoints,1)*p2'];
        end;
        tt=[tt;(tn+1)];
        theta1=theta(tt(:,1));
        v1=v(tt(:,1),tt(:,1));
        z1=[z1,p1];
        zt=z1';
        mu1=logit(z1*theta1);
        se1=sqrt(mu1.*(1-mu1).*sum((v1*zt).*zt)');
        a1=mu1-se1*1.96;    b1=mu1+se1*1.96;
        tit=strcat('Figure 1',titlee,' Palm Beach County.');
        plot(xpoints,mu1,xpoints,a1,xpoints,b1,xpoints,xpoints);
        title(tit);
        xlim([0,1]);
        ylim([0,1]);
        saveas(gcf,strcat('Absentee',nj1,'1.eps'),'psc2');
    end;
    j1=j1+1;
end;
    
        


%*****************************************************/
% Procedures @    
function xx=xp(xs,p,j);
global xc;
nj=num2str(j);
load(strcat('dat',nj,'.mat'));
xx=xc;
if sum(xselect)>0
    xx=[xx,xs];
end;
if sum(pselect)>0
    xx=[xx,log(p(:,pselect))];
end;
if ipselect==1
    xx=[xx,(ilogit(p(:,1)*ones(1,length(xc(1,:)))).*xc)];
end;

function r=pr_hat(y,x,theta);
r=(logit(x*theta(1:length(x(1,:)))).*theta(length(theta(:,1)))+y(:,1))./(y(:,1)+y(:,2)+theta(length(theta(:,1))));

function r=expt(x);
r=exp(x.*(x>(-700)).*(x<700)-(x<=(-700))*700+(x>700)*700);
 
function r=logit(u);
r=1./(1+expt(-u));
 
function r=ilogit(y);
r=-log((1./y)-1);

%***********************************************
%BETAS.M
%Useful procedures for beta-binomial-logit likelihood analysis
%************************************************

%************************************************
%LnGamma
%Computes the natural log of the gamma function
%Uses Stirling's approximation.
%Inputs:
%	x	matrix argument
%Outputs:
%	g	ln(gamma(x))
%The elements of x should be positive. 
%For non-positive values, LnGamma returns values of "0"
%*************************************************

function g=LnGamma(x);
d=(x<=0);		% non-positives %
xd=x.*(1-d)+d;		% converts non-positives to unity %
xt=xd+2; 		% to allow use of Stirling's approximation %
x2 = xt.*xt;
x3 = xt.*x2;
x5 = x3.*x2;
x7 = x5.*x2;
x9 = x7.*x2;
% stirling's approximation %
g = 0.5 * log(2*pi) + (xt - 0.5) .* log(xt) - xt + 1./(12.*xt) - 1./(360.*x3) + 1./(1260.*x5) - 1./(1680.*x7) + 1./(1188.*x9);
g=g-log(xd.*(xd+1));	% removes the added "2" %
g=g.*(1-d);		% returns "0" for non-positives %

%************************************************
%D_LnGamma, D2_LnGamma,
%Computes the first and second derivatives of the natural log of the gamma function.
%Uses Stirling's approximation.
%The elements of x should be positive. 
%For non-positive values, the procs return values of "0"
%*************************************************/
function g=D_LnGamma(x);
d=(x<=0);		% non-positives @
xd=x.*(1-d)+d;  % converts non-positives to unity @
xt=xd+2; 		% to allow use of Stirling's approximation @
x2 = xt.*xt;
x4 = x2.*x2;
x6 = x4.*x2;
x8 = x6.*x2;
x10 = x8.*x2;
g = log(xt) - 1./(2.*xt) - 1./(12.*x2) + 1./(120.*x4) - 1./(252.*x6) + 1./(240.*x8) - 1./(132.*x10) ;
g = g - 1./xd - 1./(1+xd);	% removes the added "2" @
g = g.*(1-d);		% returns "0" for non-positives @

%*******************************************%
function g=D2_LnGamma(x);
d=(x<=0);		% non-positives @
xd=x.*(1-d)+d;		% converts non-positives to unity @
xt=xd+2; 		% to allow use of Stirling's approximation @
x2 = xt.*xt;
x3 = xt.*x2;
x5 = x3.*x2;
x7 = x5.*x2;
x9 = x7.*x2;
x11 = x9.*x2;
g = 1./xt + 1./(2.*x2) + 1./(6.*x3) - 1./(30.*x5) + 1./(42.*x7) - 1./(30.*x9) + 1./(11.*x11);
g = g + 1./(xd.^2) + 1./((1+xd).^2);  % removes the added "2" @
g = g.*(1-d);		% returns "0" for non-positives @

%************************************************
%Like_Beta
%Log Likelihood of Beta-Binomial-Logit Model

%Input:	theta	parameter vector
%Output:	negative log-likelihood

%Takes y_ x_ and w_ from the global variable table
%y_={ya,yb} are the dependent variables
%x_ 	regressor matrix
%w_ 	vector indicating inclusion of the observation. 
%	(1 for inclusion, 0 for exclusion)
	
%*************************************************/
function [spp,gr,hs]=Like_Beta(theta);
global w_;
global y_;
global x_;
w=w_;y=y_;x=x_;
y1=y(:,1);
y2=y(:,2);
mu=logit(x*theta(1:length(x(1,:))));
s=w.*theta(length(theta(:,1))); 
sl=.01; 
s=(s-sl).*(s>sl)+sl;
a=mu.*s; 
b=(1-mu).*s;
pp=LnGamma(y1+a)+LnGamma(y2+b)-LnGamma(y1+y2+s)-LnGamma(a)-LnGamma(b)+LnGamma(s);
spp=-sum(pp)';
gr=G_Like_Beta(theta);
hs=H_Like_Beta(theta);
%************************************************
%G_Like_Beta
%Gradient of Log Likelihood of Beta-Binomial-Logit Model

%Input:	theta	parameter vector
%Output:	1xk 	derivative vector evaluated at theta

%Takes y_ x_ and w_ from the global variable table
%y_={ya,yb} are the dependent variables
%x_ 	regressor matrix
%w_ 	vector indicating inclusion of the observation. 
%	(1 for inclusion, 0 for exclusion)
%*************************************************/
function gr=G_Like_Beta(theta);
global w_;
global y_;
global x_;
gr=sum(V_G_Like_Beta(theta,y_,x_));

%************************************************
%V_G_Like_Beta
%Vector Gradient of Log Likelihood of Beta-Binomial-Logit Model

%Inputs:	theta	parameter vector
%	y	dependent variables, nx2
%	x	regressor matrix
%Output:	nxk 	derivative matrix evaluated at theta

%Note: This is useful for calculation of robust covariance matrices
%Takes w_ from the global variable table
%w_ 	vector indicating inclusion of the observation. 
%	(1 for inclusion, 0 for exclusion)
%*************************************************/
function ngr=V_G_Like_Beta(theta,y,x);
global w_;
w=w_;
y1=y(:,1).*w;
y2=y(:,2).*w;
mu=logit(x*theta(1:length(x(1,:))));
s=w.*theta(length(theta));
a=mu.*s;b=(1-mu).*s;
dga=D_LnGamma(y1+a)-D_LnGamma(a);    
dgb=D_LnGamma(y2+b)-D_LnGamma(b); 
dgs=D_LnGamma(y1+y2+s)-D_LnGamma(s);
b1=(dga-dgb).*mu.*(1-mu).*s;
b2=dga.*mu+dgb.*(1-mu)-dgs;
gr=[(x.*(b1*ones(1,length(x(1,:))))),b2];
ngr=-gr;

%************************************************
%H_Like_Beta
%Hessian of Log Likelihood of Beta-Binomial-Logit Model

%Input:	theta	parameter vector
%Output:	kxk	second derivative matrix evaluated at theta

%Takes y_ x_ and w_ from the global variable table
%y_={ya,yb} are the dependent variables
%x_ 	regressor matrix
%w_ 	vector indicating inclusion of the observation. 
%	(1 for inclusion, 0 for exclusion)
%*************************************************/
function nh=H_Like_Beta(theta);
global w_;w=w_;
global y_;y=y_;y1=y(:,1);y2=y(:,2);
global x_;x=x_;rx=length(x(1,:));
mu=logit(x*theta(1:rx));
s=w.*theta(length(theta)); 
sl=.01; 
s=(s-sl).*(s>sl)+sl;
mu2=mu.*(1-mu).*s;
a=mu.*s; b=(1-mu).*s;
dga=D_LnGamma(y1+a)-D_LnGamma(a);    
dgb=D_LnGamma(y2+b)-D_LnGamma(b); 
dgs=D_LnGamma(y1+y2+s)-D_LnGamma(s);
dha=D2_LnGamma(y1+a)-D2_LnGamma(a);    
dhb=D2_LnGamma(y2+b)-D2_LnGamma(b); 
dhs=D2_LnGamma(y1+y2+s)-D2_LnGamma(s);
b11=(dga-dgb).*mu2.*(1-2*mu)+(dha+dhb).*(mu2.^2);
b21=(dha.*mu.*s-dhb.*(1-mu).*s+dga-dgb).*mu.*(1-mu);
b22=dha.*(mu.^2)+dhb.*((1-mu).^2)-dhs;
h11=x'*(x.*(b11*ones(1,length(x(1,:)))));
h21=x'*b21;
h22=sum(b22)';
h=[[h11,h21];[h21',h22]];
nh=-h;

% ************************************************ %